import { Event } from '@vscode-alt/monaco-editor/esm/vs/base/common/event';
import { Disposable } from '@vscode-alt/monaco-editor/esm/vs/base/common/lifecycle';
import { IURITransformer } from '@vscode-alt/monaco-editor/esm/vs/base/common/uriIpc';
import { IRPCProtocol, ProxyIdentifier, HostIdentifier } from './proxy-identifier';
import { VSBuffer } from '@vscode-alt/monaco-editor/esm/vs/base/common/buffer';
export interface JSONStringifyReplacer {
    (key: string, value: any): any;
}
export declare function safeStringify(obj: any, replacer: JSONStringifyReplacer | null): string;
export declare const enum RequestInitiator {
    LocalSide = 0,
    OtherSide = 1
}
export declare const enum ResponsiveState {
    Responsive = 0,
    Unresponsive = 1
}
export interface IRPCProtocolLogger {
    logIncoming(msgLength: number, req: number, initiator: RequestInitiator, str: string, data?: any): void;
    logOutgoing(msgLength: number, req: number, initiator: RequestInitiator, str: string, data?: any): void;
}
/**
 * @sri
 * Using custom protocol to provide forward functionality.
 */
export type IRPCMessagePassingProtocol = {
    onMessage: Event<VSBuffer>;
    send(buffer: VSBuffer, hostId: HostIdentifier): void;
    forward?(buffer: VSBuffer, hostId: HostIdentifier): void;
};
export declare class RPCProtocol extends Disposable implements IRPCProtocol {
    private static UNRESPONSIVE_TIME;
    private readonly _onDidChangeResponsiveState;
    readonly onDidChangeResponsiveState: Event<ResponsiveState>;
    private readonly _protocol;
    private readonly _logger;
    private readonly _uriTransformer;
    private readonly _uriReplacer;
    private _isDisposed;
    private readonly _locals;
    private readonly _proxies;
    private _lastMessageId;
    private readonly _cancelInvokedHandlers;
    private readonly _pendingRPCReplies;
    private _responsiveState;
    private _unacknowledgedCount;
    private _unresponsiveTime;
    private _asyncCheckUresponsive;
    private localHostId;
    constructor(protocol: IRPCMessagePassingProtocol, _localHostId?: HostIdentifier | null, logger?: IRPCProtocolLogger | null, transformer?: IURITransformer | null);
    dispose(): void;
    private _onWillSendRequest;
    private _onDidReceiveAcknowledge;
    private _checkUnresponsive;
    private _setResponsiveState;
    get responsiveState(): ResponsiveState;
    transformIncomingURIs<T>(obj: T): T;
    getProxy<T>(identifier: ProxyIdentifier<T>): T;
    private _createProxy;
    set<T, R extends T>(identifier: ProxyIdentifier<T>, value: R): R;
    assertRegistered(identifiers: ProxyIdentifier<any>[]): void;
    private _receiveOneMessage;
    private _receiveRequest;
    private _receiveCancel;
    private _receiveReply;
    private _receiveReplyErr;
    private _invokeHandler;
    private _doInvokeHandler;
    private _remoteCall;
}
export declare class MessageBuffer {
    static alloc(type: MessageType, req: number, messageSize: number): MessageBuffer;
    static read(buff: VSBuffer, offset: number): MessageBuffer;
    private _buff;
    private _offset;
    get buffer(): VSBuffer;
    private constructor();
    static sizeUInt8(): number;
    writeUInt8(n: number): void;
    readUInt8(): number;
    writeUInt32(n: number): void;
    readUInt32(): number;
    static sizeShortString(str: VSBuffer): number;
    writeShortString(str: VSBuffer): void;
    readShortString(): string;
    static sizeLongString(str: VSBuffer): number;
    writeLongString(str: VSBuffer): void;
    readLongString(): string;
    static sizeBuffer(buff: VSBuffer): number;
    writeBuffer(buff: VSBuffer): void;
    readBuffer(): Buffer;
    static sizeVSBuffer(buff: VSBuffer): number;
    writeVSBuffer(buff: VSBuffer): void;
    readVSBuffer(): VSBuffer;
    static sizeMixedArray(arr: VSBuffer[], arrType: ArgType[]): number;
    writeMixedArray(arr: VSBuffer[], arrType: ArgType[]): void;
    readMixedArray(): Array<string | Buffer | VSBuffer>;
}
export declare class MessageIO {
    private static _arrayContainsBuffer;
    static serializeRequest(req: number, rpcId: number, method: string, args: any[], usesCancellationToken: boolean, replacer: JSONStringifyReplacer | null): VSBuffer;
    private static _requestJSONArgs;
    static deserializeRequestJSONArgs(buff: MessageBuffer): {
        rpcId: number;
        method: string;
        args: any[];
    };
    private static _requestMixedArgs;
    static deserializeRequestMixedArgs(buff: MessageBuffer): {
        rpcId: number;
        method: string;
        args: any[];
    };
    static serializeAcknowledged(req: number): VSBuffer;
    static serializeCancel(req: number): VSBuffer;
    static serializeReplyOK(req: number, res: any, replacer: JSONStringifyReplacer | null): VSBuffer;
    private static _serializeReplyOKEmpty;
    private static _serializeReplyOKBuffer;
    private static _serializeReplyOKVSBuffer;
    static deserializeReplyOKBuffer(buff: MessageBuffer): Buffer;
    static deserializeReplyOKVSBuffer(buff: MessageBuffer): VSBuffer;
    private static _serializeReplyOKJSON;
    static deserializeReplyOKJSON(buff: MessageBuffer): any;
    static serializeReplyErr(req: number, err: any): VSBuffer;
    private static _serializeReplyErrEror;
    static deserializeReplyErrError(buff: MessageBuffer): Error;
    private static _serializeReplyErrEmpty;
}
export declare const enum MessageType {
    RequestJSONArgs = 1,
    RequestJSONArgsWithCancellation = 2,
    RequestMixedArgs = 3,
    RequestMixedArgsWithCancellation = 4,
    Acknowledged = 5,
    Cancel = 6,
    ReplyOKEmpty = 7,
    ReplyOKVSBuffer = 8,
    ReplyOKJSON = 9,
    ReplyOKJSONWithBuffers = 10,
    ReplyErrError = 11,
    ReplyErrEmpty = 12
}
export declare const enum ArgType {
    String = 1,
    Buffer = 2,
    VSBuffer = 3
}
